import logging

from rest_framework import serializers
from .generic import DynamicFieldsModelSerializer
from util.factories import GradyUserFactory
from core import models
from django.core import exceptions

import django.contrib.auth.password_validation as validators

log = logging.getLogger(__name__)
user_factory = GradyUserFactory()


class TutorSerializer(DynamicFieldsModelSerializer):
    feedback_created = serializers.SerializerMethodField()
    feedback_validated = serializers.SerializerMethodField()
    password = password = serializers.CharField(
        style={'input_type': 'password'},
        write_only=True,
        required=False
    )

    @staticmethod
    def _get_completed_assignments(obj):
        return models.TutorSubmissionAssignment.objects.filter(
            is_done=True,
            subscription__owner=obj,
        )

    def get_feedback_created(self, obj):
        return self._get_completed_assignments(obj).filter(
            subscription__feedback_stage=models.SubmissionSubscription.FEEDBACK_CREATION  # noqa
        ).count()

    def get_feedback_validated(self, obj):
        return self._get_completed_assignments(obj).filter(
            subscription__feedback_stage=models.SubmissionSubscription.FEEDBACK_VALIDATION  # noqa
        ).count()

    def create(self, validated_data) -> models.UserAccount:
        log.info("Crating tutor from data %s", validated_data)
        return user_factory.make_tutor(
            username=validated_data['username'],
            password=validated_data.get('password'),
            is_active=validated_data.get('is_active', False))

    def validate(self, data):
        user = models.UserAccount(**data)
        password = data.get('password')

        try:
            if password is not None:
                validators.validate_password(password=password, user=user)
        except exceptions.ValidationError as err:
            raise serializers.ValidationError({'password': list(err.messages)})
        return data

    class Meta:
        model = models.UserAccount
        fields = ('pk',
                  'password',
                  'is_active',
                  'username',
                  'feedback_created',
                  'feedback_validated')
